// Wuerfel mit AtTiny
// 
// Der Wuerfel hat nur eine Taste: Taste1
// Die Taste weckt den Micro aus dem Tiefschlaf
// Die 7 LEDs zeigen das Wurfergebnis an (5x blinkend), dann geht der Micro wieder in dne Tiefschlaf
//
// V1.0 - 14.10.2019
// Grundgeruest mit (Pseudo-)Zufallszahlen und Stromsparmodi implementiert
// Problem: Nach PowerOn kommen immer die gleichen Zahlen in identischer Reihenfolge
//
// V2.0 - 16.10.2019
// Optimierung des Code um Platz fuer weitere Funktionen zu bekommen -> Randomize Zufallszahlen
// Zufallszahlen randomisieren: 
//   Beim ersten Tastendruck nach dem PowerOn wird dessen Dauer "gemessen" (i++ in while-Schleife mit 9,6 MHz).
//   Mit dieser Dauer wird der Zufallszahlengenerator einmalig initialisiert.
//   Da der Micro in Pausen nur schleaft, wird der Zufallsgenerator weitergefhrt, die Zahlenfolge
//   ist also "echt zufllig" bis zum naechsten PowerDown (Batterie alle). Mit neuer Batterie
//   wird dann wieder die Tastendruckdauer gemessen, usw. Sollte schoen zufaellig sein :-)
// 17.10.2019
// Internen PullUp am Taster wieder aktiviert. Grund fuer das Problem mit den IRQs war die Methode,
//   mit der ich die LEDs ausgeschaltet habe. PORTB = 0 hat immer einen Software-IRQ ausgeloest.
// -------------------------------------------------------------------------------------------------------------

// wenn ZAHLSOFORT defined ist, wird sofort beim druecken der Taste die gewurfelte Zahl angezeigt
// Anderenfalls wird die Zahl erst nach Loslassen der Taste angezeigt
//#define ZAHLSOFORT


#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/wdt.h>

// -------------------------------------------------------------------------------------------------------------
//   LED-Gruppen       LED-#         Port-Bit
//   +---------+	+---------+    +---------+
//   | 2  4  3 |	| 5  1  4 |    | 2  4  1 |		
//   |    1    |	|    7    |    |    3    |
//   | 3  4  2 |	| 3  2  6 |    | 1  4  2 |
//   +---------+	+---------+    +---------+
// 
// -------------------------------------------------------------------------------------------------------------

// defines fuer AtTiny
#define LED7	3		//  LED Gruppe 1 - 7
#define LED5u6	2		//  LED-Gruppe 2 - 5+6
#define LED3u4	1		//  LED-Gruppe 3 - 3+4
#define LED1u2	4		//  LED-Gruppe 4 - 1+2
#define Taste 0			//	Taster
#define Reset 5			//	Reset

bool TasteAktiv = false;					// Merker ob Taste gedrueckt wurde
unsigned long uTastenDauer = 0;				// Dauer des Tastendrucks
unsigned long uTastenSet = 0;				// Zeitpunkt des Tastendrucks
unsigned char cAugenzahl;					// gewuerfelte Augenzahl
unsigned int i = 0;							// Randomizer Variable
bool randGelaufen = false;					// Rondomizer nur einmal nach PowerOn durchlaufen

// Optimierungen
#undef digitalRead()
#define digitalRead(pinnummer) ((PINB) & (1 << (pinnummer)))

// Interrruptroutine fuer Taste
ISR(PCINT0_vect)
{
	// Die Routine muss nichts tun, nur den Micro aufwecken
	// Nach wake up geht es im Programmcode nach "sleep_mode()" weiter
}

// -------------------------------------------------------------------------------------------------------------
void setup()
{
	// IOs - Richtung festlegen
	pinMode(LED1u2, OUTPUT); pinMode(LED3u4, OUTPUT); pinMode(LED5u6, OUTPUT); pinMode(LED7, OUTPUT);
	pinMode(Taste, INPUT_PULLUP);
	//pinMode(Reset, INPUT);				// unnoetig, ist default

	// PCI triggert auf jedem Pin und bei jeder Flanke
	// - Pins festlegen (PCMSK) 
	// - Pegel nach IRQ testen um fallende Flanke zu erkennen (Pins haben PullUps -> aktiv low)
	// alle Pins auf Interrupt einstellen
	bitSet(GIMSK, PCIE);					// Pin Change Interrupt Enable fuer P0..P5
	bitSet(PCMSK, PCINT0);					// PCINT auf Taste (P0) enable, bei jeder Flanke wird ein IRQ erzeugt

	sei();						// Interrupts einschalten damit Aufwecken funktioniert

	// Use the Power Down sleep mode
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);

	// Analoge Module ausschalten
	bitSet(ACSR, ACD);			// power off analog comparator
	bitClear(ADCSRA, ADEN);		// disable AD converter
	bitClear(MCUSR, WDRF);		// Watchdog Reset Flag loeschen (auch ohne WD enabled ausfuehren lt. Datenblatt)

	sleep_mode();				//gehe in sleep und warte auf einen Interrupt...

}

// -------------------------------------------------------------------------------------------------------------
void loop()
{
	//unsigned int i;						// Randomizer Variable

	// die nachfolgende Mimik sendet den Impuls nur einmal je Tastendruck - Taste
	if ((digitalRead(Taste) == LOW) && (TasteAktiv == false))
	{
		TasteAktiv = true;										// Trigger merken

		if (randGelaufen == false)								// nur einmalig randomisieren
		{
			while (digitalRead(Taste) == LOW)					// Randomizer Zahl ueber Tastendruckzeit ermitteln
			{
				i++;
			}

			srand(i);											// Zufallszahlengenerator randomisieren
			randGelaufen = true;
		}

		cAugenzahl = (rand() % 6) + 1;							// Zahlen zwischen 1 und 6

#if defined ZAHLSOFORT
		showResult();											// Augen anzeigen
#endif
	}
	else if ((TasteAktiv == true) && (digitalRead(Taste) == HIGH))
	{
#ifndef ZAHLSOFORT
		showResult();											// Augen anzeigen
#endif

		TasteAktiv = false;										// Trigger loeschen

		// erst wieder in Sleep gehen, wenn die Taste losgelassen wurde 
		//  (verhindern, dass die Flanke beim Loslassen ebenfalls einen IRQ triggert)
		// der Micro faellt hier in den deep sleep mode und wartet auf einen Tastendruck
		bitSet(MCUCR, SE);										// Sleep Enable einschalten
		sleep_mode();											// gehe in sleep und warte auf einen Interrupt...
		// nach Wakeup geht es hier weiter
		bitClear(MCUCR, SE);									// Sleep Enable ausschalten
	}
}
// Augenzahl anzeigen
void showResult(void)
{
	for (char i = 0; i < 5; i++)
	{
		switch (cAugenzahl)
		{
		case 1:
			//bitSet(PORTB, LED7);
			PORTB |= B00001000;							// LED7 - Bit 3;
			break;
		case 2:
			//bitSet(PORTB, LED3u4);
			PORTB |= B00000010;							// LED3u4 - Bit 1;
			break;
		case 3:
			//bitSet(PORTB, LED7);
			//bitSet(PORTB, LED3u4);
			PORTB |= B00001010;							// (LED3u4 & LED7 - Bits 1 & 3);
			break;
		case 4:
			//bitSet(PORTB, LED3u4);
			//bitSet(PORTB, LED5u6);
			PORTB |= B00000110;							// (LED3u4 & LED7 - Bits 1 & 2);
			break;
		case 5:
			//bitSet(PORTB, LED3u4);
			//bitSet(PORTB, LED5u6);
			//bitSet(PORTB, LED7);
			PORTB |= B00001110;							// (LED3u4 & LED5u6 & LED7 - Bits 1 & 2 & 3);
			break;
		case 6:
			//bitSet(PORTB, LED3u4);
			//bitSet(PORTB, LED5u6);
			//bitSet(PORTB, LED1u2);
			PORTB |= B00010110;							// (LED3u4 & LED5u6 & LED1u2 - Bits 1 & 2 & 4);
			break;
		default:
			//PORTB = 0;
			PORTB &= ~(B00011110);
		}
		delay(200);

		PORTB &= ~(B00011110);
		delay(200);
	}
}